from functools import partial
from typing import Literal

from cow.manager.process import SanicProcess
from cow.manager.utils import createManager
from cow.web.configs_handler import ConfigsHandler

from .base_app import BaseLambdaApp as LambdaApp
from .configs.configs.services import SettingsLambdaUnit
from .handlers import Handler, HealthCheckHandler, SpecDocHandler
from .helpers import getChecks4HandlersLambda, getChecks4StandaloneLambda, getParsedCMDArgs

try:
    from lambda_main import UserCtx
except ImportError:

    class UserCtx:
        """Empty user application context"""

        ...


def _runLambda(lambdaType: Literal["standalone", "handlers"]):
    """Run lambda"""
    cmdArgs = getParsedCMDArgs()
    application = LambdaApp(
        serviceName=cmdArgs.name, lambdaVersion=cmdArgs.version, lambdaType=lambdaType, userCtxCls=UserCtx
    )
    application.add_route(Handler.as_view(), "/main")
    application.add_route(HealthCheckHandler.as_view(), "/healthcheck")
    application.add_route(SpecDocHandler.as_view(), "/docs/spec")
    application.sanic.add_route(ConfigsHandler.as_view(), "/config", strict_slashes=True)
    application.run()


def run(lambdaType: Literal["standalone", "handlers"]):
    """Run lambda using process manager"""
    target = partial(_runLambda, lambdaType=lambdaType)
    processes = [SanicProcess(name="lambda", target=target)]
    checksGetter = getChecks4HandlersLambda if lambdaType == "handlers" else getChecks4StandaloneLambda
    manager = createManager(processes, configClass=SettingsLambdaUnit, getChecks=checksGetter, port=8000)

    manager.run()
